//pin names wrt to SCMP i.e. RX = SCMP in, Arduino out
#define PIN_SCMP_RXRELAY 2
#define PIN_SCMP_RX 3
#define PIN_SCMP_TX 5
#define PIN_PIP 7

#define PRESCALER 8
#define BAUD      220
#define T1_OCR    (F_CPU/(BAUD*4*PRESCALER))

void setup() {

  pinMode(PIN_SCMP_RXRELAY, INPUT);
  pinMode(PIN_SCMP_RX, OUTPUT);
  digitalWrite(PIN_SCMP_RX, HIGH);
  pinMode(PIN_SCMP_TX, INPUT);
  pinMode(PIN_PIP, OUTPUT);

  
  //usb serial
  Serial.begin(9600);

  //Leonardo wait for serial
  while (!Serial)
  { //nowt
  }

  Serial.println("Ready!");

  // initialize timer1 
  noInterrupts();                           // disable all interrupts
  TCCR1A = 0;
  TCCR1B = (1 << WGM12) | (1 << CS11);      // CTC mode
  TCNT1  = 0;
  OCR1A = T1_OCR;
  TIMSK1 |= (1 << OCIE1A);   // enable timer overflow interrupt
  interrupts();             // enable all interrupts

}

#define TX_STATE_WAIT_START 0
#define TX_STATE_ACT 1
#define TX_STATE_STOP 2

//tx variables read in from SCMP TX port
volatile unsigned char tx_state = TX_STATE_WAIT_START;    //state of TX state machine
volatile unsigned char tx_char_ready = 0;                 //a char is ready for the main loop
volatile unsigned char tx_char;                           //the last char read (only valid when tx_char_ready
volatile unsigned char tx_cntdn;                          //number of cycles to next bit
volatile unsigned char tx_ix;                             //number of bits left

#define RX_STATE_WAIT_START 0
#define RX_STATE_ACT 1
#define RX_STATE_STOP 2
#define RX_STATE_STOP2 3

volatile unsigned char rx_state = RX_STATE_WAIT_START;
volatile unsigned char rx_char_ready = 0;
volatile unsigned char rx_char;
volatile unsigned char rx_cntdn;
volatile unsigned char rx_ix;

ISR(TIMER1_COMPA_vect)       // interrupt service routine 
{

  if (tx_state == TX_STATE_WAIT_START && !tx_char_ready) {
    if (digitalRead(PIN_SCMP_TX)) {
      tx_ix = 8;
      tx_cntdn = 6;     // start bit + 1/2 of next bit delay
      tx_state = TX_STATE_ACT;
    }
  } else if (tx_state == TX_STATE_ACT) {
    if (--tx_cntdn <= 0) {
      tx_char = tx_char >> 1;
      tx_char |= (digitalRead(PIN_SCMP_TX) ? 0 : 0x80);
      tx_cntdn = 4;
      if (--tx_ix <= 0) {
        tx_state = TX_STATE_STOP;
        tx_char_ready = 1;       
      }
    }
  } else {
    if (--tx_cntdn <= 0) {
      tx_state = TX_STATE_WAIT_START;
    }    
  }

  if (rx_state == RX_STATE_WAIT_START && rx_char_ready) {
    rx_ix = 8;
    rx_cntdn = 4;     // start bit + 1/2 of next bit delay
    rx_state = RX_STATE_ACT;
    digitalWrite(PIN_SCMP_RX, LOW); //start bit
  } else if (rx_state == RX_STATE_ACT) {
    if (--rx_cntdn <= 0) {
      digitalWrite(PIN_SCMP_RX, (rx_char & 1));
      rx_char = rx_char >> 1;
      digitalWrite(PIN_PIP, !digitalRead(PIN_PIP));
      rx_cntdn = 4;
      if (--rx_ix <= 0) {
        rx_state = RX_STATE_STOP;
        rx_cntdn = 4;
      }
    }
  } else if (rx_state == RX_STATE_STOP) {
    if (--rx_cntdn <= 0) {
      digitalWrite(PIN_SCMP_RX, HIGH);  //long stop bit
      rx_state = RX_STATE_STOP2;
      rx_cntdn = 8;
    }
  } else {
    if (--rx_cntdn <= 0) {
      rx_char_ready = 0;       
      rx_state = RX_STATE_WAIT_START;
    }
  }

}

void loop() {

  if (tx_char_ready) {
    Serial.write(tx_char & 0x7F);
    tx_char_ready = 0;
  }

  if (rx_char_ready == 0) {
    if (Serial.available() > 0) {
      rx_char = Serial.read();
      rx_char_ready = 1;
    }
  }
  
  return;
  /*
  while (1) {
    //check to see if there's a char waiting in from PC an if the relay is on
    //if (digitalRead(PIN_SCMP_RXRELAY) && charwaiting) {
    if (charwaiting) {
      digitalWrite(PIN_PIP, HIGH);
      //send a character
      digitalWrite(PIN_SCMP_RX, LOW); //start bit
      delayMicroseconds(4 * MICRO_DLY);
      for (int i = 0 ; i < 8; i++) {
        digitalWrite(PIN_SCMP_RX, (waitingchar & 1));
        waitingchar = waitingchar >> 1;
        delayMicroseconds(4 * MICRO_DLY);
      }
      digitalWrite(PIN_SCMP_RX, HIGH);  //long stop bit
      delayMicroseconds(12 * MICRO_DLY);
      charwaiting = 0;
      digitalWrite(PIN_PIP, LOW);
    }


    if (digitalRead(PIN_SCMP_TX)) {
      //receive character
      //assume we are at or near start of stop bit, wait 1.5 bit times
      delayMicroseconds(6 * MICRO_DLY);
      unsigned char c = 0;
      for (int i = 0; i < 8; i++) {
        c = (c >> 1) | (digitalRead(PIN_SCMP_TX) ? 0 : 0x80);
        delayMicroseconds(4 * MICRO_DLY);
      }
      Serial.write(c);
    }

    if (!charwaiting && (Serial.available() > 0)) {
      digitalWrite(PIN_PIP, HIGH);
      waitingchar = Serial.read();
      charwaiting = 1;
      digitalWrite(PIN_PIP, LOW);
    }
  }
  */
}
